<?php

namespace Bydedi\Security;

use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpFoundation\Session\SessionInterface;

class Authenticate implements AuthenticateInterface
{

    /**
     * @var UserInterface
     */
    protected $user;

    /**
     * @var SessionInterface
     */
    protected $session;

    /**
     * @var Request
     */
    protected $request;

    public function __construct(UserInterface $user, RequestStack $requestStack)
    {
        $this->user = $user;
        $this->request = $requestStack->getMasterRequest();
        if ($this->request instanceof Request) {
            $this->session = $this->request->getSession();
        }
    }

    /**
     * Check if user is authenticated
     *
     * @return bool
     */
    public function check()
    {
        return !is_null($this->getUser());
    }

    /**
     * Return current authenticated user
     *
     * @return \Bydedi\Security\Model\User
     */
    public function getUser()
    {
        if ($this->user->isLoaded()) {
            return $this->user;
        }

        $id = $this->session->get($this->getName());

        $user = null;

        if (!is_null($id)) {
            $user = $this->user->findById($id);
        }

        return $this->user = $user;
    }

    /**
     * Logs in user
     *
     * @param UserInterface $user
     * @return mixed
     */
    public function login(UserInterface $user)
    {
        $this->session->set($this->getName(), $user->id);

        $rolesNames = $user->getRolesNames();

        $this->session->set('roles', $rolesNames);
    }

    /**
     * Authenticate user
     *
     * @param array $credentials
     * @return bool
     */
    public function auth(array $credentials, $login = true)
    {
        $user = $this->user->findByLogin($credentials['login']);

        if (!is_null($user)) {
            $passwordValid = $this->user->validatePassword($credentials['password'], $user->password);

            if ($passwordValid) {
                if ($login) {
                    $this->login($user);
                }
                return true;
            }
        }

        return false;
    }

    /**
     * Deauthenticate user
     *
     * @return void
     */
    public function logout()
    {
        $this->session->remove($this->getName());
        $this->session->remove('roles');
    }

    /**
     * Unique identifier for the auth session value.
     * @return string
     */
    public function getName()
    {
        return 'user_id';
    }
}
